#include "GPIOSetup.h"
 
 /*************************************************************************
* Set a pin to be a GPIO and set it to be an input or output             *
**************************************************************************/
void SetGPIOFunction(Pio *Port, unsigned char Pin, unsigned char Function)
{

    unsigned long PortPattern;

    PortPattern = (1 << Pin);

    Port->PIO_PER = PortPattern;
    if (Function == INPUT)
        Port->PIO_ODR = PortPattern;
    if (Function == OUTPUT)
        Port->PIO_OER = PortPattern;
    
}

/**************************************************************************
* Set a GPIO pin to be either high or low.                                *
***************************************************************************/
void SetGPIO(Pio *Port, unsigned char Pin, unsigned char HighOrLow)
{

    unsigned long PortPattern;

    PortPattern = (1 << Pin);
    if (HighOrLow == HIGH)
        Port->PIO_SODR = PortPattern;
    else if (HighOrLow == LOW)
        Port->PIO_CODR = PortPattern;


}

/**********************************************************
* Set what peripheral is assigned to a pin                *
* Each pin has two registers that set the peripheral that *
* is assigned to the pin. PIO_ABCDSR[0] & PIO_ABCDSR[1]   *
* These two registers use bit 0 to bit 31 to assign one   *
* of four functions. Peripheral A = 0, through to         *
* peripheral D = 3                                        *
**********************************************************/
unsigned char SetGPIOPeripheral(Pio *Port, unsigned char PinNumber, unsigned char Peripheral)
{

    unsigned long PeripheralBitPattern[2];
    unsigned long PeripheralBitPatternMask;

// First set the ABCDSR registers to 0, set up a mask with all values to 1 other than the pin being
// set up
    PeripheralBitPatternMask = ~(1 << PinNumber);
    
    Port->PIO_ABCDSR[0] &= PeripheralBitPatternMask;
    Port->PIO_ABCDSR[1] &= PeripheralBitPatternMask;

// Now set up the bit pattern appropriate for the 
    PeripheralBitPattern[0] = 0;
    PeripheralBitPattern[1] = 0;

    if (Peripheral == PERIPHERAL_B)
        {
        PeripheralBitPattern[0] = (1 << PinNumber);
        }
    else if (Peripheral == PERIPHERAL_C)
        {
        PeripheralBitPattern[1] = (1 << PinNumber);
        }
    else if (Peripheral == PERIPHERAL_D)
        {
        PeripheralBitPattern[0] = (1 << PinNumber);
        PeripheralBitPattern[1] = (1 << PinNumber);
        }
     else if (Peripheral != PERIPHERAL_A)
        return ERROR;

    Port->PIO_ABCDSR[0] |= PeripheralBitPattern[0];
    Port->PIO_ABCDSR[1] |= PeripheralBitPattern[1];

    PeripheralBitPatternMask = (1 << PinNumber);

    Port->PIO_PDR = PeripheralBitPatternMask;

    return SUCCESS;

}


