#ifndef DISK_IO_H
#define DISK_IO_H


#include "defines.h"

//#include "main.h"
#include "integer.h"
#include "sam3sd8.h"
#include "GPIOSetup.h"

/* Status of Disk Functions */
typedef BYTE	DSTATUS;

/* Results of Disk Functions */
typedef enum {
	RES_OK = 0,		/* 0: Successful */
	RES_ERROR,		/* 1: R/W Error */
	RES_WRPRT,		/* 2: Write Protected */
	RES_NOTRDY,		/* 3: Not Ready */
	RES_PARERR		/* 4: Invalid Parameter */
} DRESULT;


/*---------------------------------------*/
/* Prototypes for disk control functions */

int assign_drives (int, int);
DSTATUS disk_initialize (BYTE);
DSTATUS disk_status (BYTE);
DRESULT disk_read (BYTE, BYTE*, DWORD, BYTE);
DRESULT disk_write (BYTE drv, BYTE *buff, DWORD sector, BYTE count);
DRESULT disk_ioctl (BYTE, BYTE, void*);



/* Disk Status Bits (DSTATUS) */

#define STA_NOINIT		0x01	/* Drive not initialized */
#define STA_NODISK		0x02	/* No medium in the drive */
#define STA_PROTECT		0x04	/* Write protected */


//define error masks
#define MCI_ERRORS_MASK	(HSMCI_SR_RINDE | HSMCI_SR_RDIRE   \
| HSMCI_SR_RENDE | HSMCI_SR_RTOE | HSMCI_SR_DCRCE \
| HSMCI_SR_DTOE | HSMCI_SR_OVRE | HSMCI_SR_UNRE)

//don't care argument
#define ARG_DONT_CARE		0x00000000



/* Generic command (mandatory for FatFs) */
#define CTRL_SYNC			0	/* Flush disk cache (for write functions) */
#define GET_SECTOR_COUNT	1	/* Get media size (for only f_mkfs()) */
#define GET_SECTOR_SIZE		2	/* Get sector size (for multiple sector size (_MAX_SS >= 1024)) */
#define GET_BLOCK_SIZE		3	/* Get erase block size (for only f_mkfs()) */

unsigned char SendCommand(unsigned long Command, unsigned long Argument, unsigned long *Response);
unsigned char ReadBlock(unsigned char *Buff, unsigned long StartBlock);
unsigned char InitiliaseCard(void);
void ReadBlockWrapper(unsigned char *Buffer,unsigned long StartAddress);
void GetRCA(unsigned long *Data);
unsigned char SendExtendedCommand(unsigned long Command, unsigned long CardNumber, unsigned long Argument, unsigned long *Response);
unsigned long GetHSMCIClockDivisionForRequiredSpeed(unsigned long ProcessorSpeed, unsigned long RequiredHSMCIClockSpeed);
unsigned long GetCardType(void);
unsigned long ReadSDSector(unsigned char *Buffer, unsigned long lba);
unsigned char InitialiseSD(void);
DRESULT disk_ioctl (BYTE drv,BYTE ctrl,void *buff);
DRESULT disk_read(BYTE drv, BYTE *buff, DWORD sector, BYTE count);
void delay_ms(unsigned long DelayTime);
void delay_us(unsigned long DelayTime);

#define PASS			0
#define FAIL			1

// 512 byyes is almost always used, but it would be better to not have this as a #define and take it from the card
#define BLOCK_SIZE			512

// Maximum number of blocks to be written at one time. 
#define MAX_DMA_BLOCKS		127

#define CMD0 0
#define CMD1 1
#define CMD2 2
#define CMD3 3
#define CMD5 5
#define CMD7 7
#define CMD8 8
#define CMD9 9
#define CMD13 13
#define CMD17 17
#define CMD24 24
#define CMD55 55
#define CMD58 58

#define ACMD6 6
#define ACMD41 41

#define ZERO_LENGTH_RESPONSE 0
#define FORTY_EIGHT_BIT_LENGTH_RESPONSE 1
#define ONE_HUNDRED_AND_THIRTY_SIX_BIT_LENGTH_RESPONSE 2
#define FORTY_EIGHT_BIT_LENGTH_RESPONSE_FOR_DATA_READ_TRANSFER 3
#define FORTY_EIGHT_BIT_LENGTH_RESPONSE_FOR_DATA_WRITE_TRANSFER 4
#define ERROR_INVALID_COMMAND 0xC0

#define SD_BUS_SPEED 20000000

#define HD_CARD 1
#define SD_CARD 0



#endif 